@@ -97,6 +97,7 @@ for mod_name in ['sherpa.utils._utils',
                  'sherpa.astro.utils._utils',
                  'sherpa.astro.utils._pileup',
                  'sherpa.astro.utils._region',
+                 'sherpa.astro.utils._wcs',
                  'sherpa.astro.models._modelfcts',
                  'sherpa.astro.io.pyfits_backend',
                  'sherpa.image.DS9',
@@ -154,7 +155,8 @@ extensions = [
     'sphinx_astropy.ext.intersphinx_toggle',
     'sphinx_astropy.ext.edit_on_github',
     # notebooks
-    'nbsphinx'
+    'nbsphinx',
+    'sphinx_rtd_theme',
 ]
 # Imported from sphinx_astropy so we don't have to maintain the list
 # of servers
@@ -286,7 +288,7 @@ main_doc = 'index'
 
 # General information about the project.
 project = 'Sherpa'
-copyright = '2019-2021, Chandra X-ray Center, Smithsonian Astrophysical Observatory.'
+copyright = '2019-2022, Chandra X-ray Center, Smithsonian Astrophysical Observatory.'
 author = 'Chandra X-ray Center, Smithsonian Astrophysical Observatory'
 
 # The version info for the project you're documenting, acts as replacement for
@@ -615,7 +617,7 @@ def setup(app):
     #
     # Guessing at rebuild and types arguments.
     #
-    githash = sherpa._version.get_versions()['full']
+    githash = sherpa._version.get_versions()['full-revisionid']
     if githash.endswith('.dirty'):
         githash = githash[:-6]
 
