@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2007, 2014, 2015, 2016, 2019, 2020, 2021
+#  Copyright (C) 2007, 2014, 2015, 2016, 2019, 2020, 2021, 2022
 #  Smithsonian Astrophysical Observatory
 #
 #
@@ -35,6 +35,7 @@ the standard subpackages, use ``import sherpa.all`` or
 """
 
 import datetime
+import importlib
 import logging
 import os
 import os.path
@@ -44,9 +45,8 @@ import sys
 
 __all__ = ('citation', 'get_config', 'get_include', 'smoke')
 
-from ._version import get_versions
-__version__ = get_versions()['version']
-del get_versions
+from . import _version
+__version__ = _version.get_versions()['version']
 
 
 class Formatter(logging.Formatter):
@@ -744,30 +744,27 @@ def get_config():
 
     filename = "sherpa-standalone.rc"
 
-    home_dir = None
-    config = None
+    # The behavior depends on whether the NOSHERPARC
+    # environment variable is set.
+    #
+    if 'NOSHERPARC' not in os.environ:
 
-    # If NOSHERPARC is set, read in system config file
-    # ignore any user config file
-    if 'NOSHERPARC' in os.environ:
-        return os.path.join(os.path.dirname(__file__), filename)
+        # If SHERPARC is set, try that
+        #
+        if 'SHERPARC' in os.environ:
+            config = os.environ.get('SHERPARC')
+            if os.path.isfile(config):
+                return config
 
-    # If SHERPARC is set, read in config file from there,
-    # and ignore default location
-    if 'SHERPARC' in os.environ:
-        config = os.environ.get('SHERPARC')
-        if os.path.isfile(config):
-            return config
+        # Can we use the HOME directory?
+        #
+        home_dir = os.environ.get('HOME')
+        if home_dir is not None:
+            config = os.path.join(home_dir, f'.{filename}')
+            if os.path.isfile(config):
+                return config
 
-    # SHERPARC was not set, so look for .sherpa.rc in default
-    # location, which is user's home directory.
-    home_dir = os.environ.get('HOME')
-    config = os.path.join(home_dir, '.'+filename)
-
-    if os.path.isfile(config):
-        return config
-
-    # If no user config file is set, fall back to system config file
+    # Fall back to the system config file
     return os.path.join(os.path.dirname(__file__), filename)
 
 
@@ -856,14 +853,14 @@ def _install_test_deps():
 
     for dep in deps:
         try:
-            __import__(dep)
+            importlib.import_module(dep)
         except ImportError:
             install(dep)
 
     for plugin_name in pytest_plugins:
         module = plugin_name.replace("-", "_")
         try:
-            __import__(module)
+            importlib.import_module(module)
         except ImportError:
             install(plugin_name)
             installed_plugins.append(module)
