@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2011, 2015, 2016, 2017, 2018, 2019, 2020, 2021
+#  Copyright (C) 2011, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022
 #  Smithsonian Astrophysical Observatory
 #
 #
@@ -42,7 +42,6 @@ import os
 import warnings
 
 import numpy
-from numpy.compat import basestring
 
 from astropy.io import fits
 from astropy.io.fits.column import _VLF
@@ -64,9 +63,10 @@ except ImportError:
     warning('failed to import WCS module; WCS routines will not be ' +
             'available')
 
-__all__ = ('get_table_data', 'get_image_data', 'get_arf_data', 'get_rmf_data',
-           'get_pha_data', 'set_table_data', 'set_image_data', 'set_pha_data',
-           'get_column_data', 'get_ascii_data')
+__all__ = ('get_table_data', 'get_header_data', 'get_image_data',
+           'get_column_data', 'get_ascii_data',
+           'get_arf_data', 'get_rmf_data', 'get_pha_data',
+           'set_table_data', 'set_image_data', 'set_pha_data')
 
 
 def _has_hdu(hdulist, name):
@@ -77,22 +77,19 @@ def _has_hdu(hdulist, name):
     return True
 
 
-def _has_key(hdu, name):
-    return name in hdu.header
-
-
 def _try_key(hdu, name, fix_type=False, dtype=SherpaFloat):
-    if _has_key(hdu, name):
+
+    try:
         key = hdu.header[name]
+    except KeyError:
+        return None
 
-        if str(key).find('none') != -1:
-            return None
+    if str(key).find('none') != -1:
+        return None
 
-        if fix_type:
-            key = dtype(key)
-        return key
-
-    return None
+    if fix_type:
+        key = dtype(key)
+    return key
 
 
 def _require_key(hdu, name, fix_type=False, dtype=SherpaFloat):
@@ -121,10 +118,15 @@ def _get_wcs_key(hdu, key0, key1, fix_type=False, dtype=SherpaFloat):
     ().
     """
 
-    if _has_key(hdu, key0) and _has_key(hdu, key1):
-        return numpy.array([_try_key(hdu, key0, fix_type, dtype),
-                            _try_key(hdu, key1, fix_type, dtype)], dtype)
-    return ()
+    val1 = _try_key(hdu, key0, fix_type, dtype)
+    if val1 is None:
+        return ()
+
+    val2 = _try_key(hdu, key1, fix_type, dtype)
+    if val2 is None:
+        return ()
+
+    return numpy.array([val1, val2], dtype)
 
 
 def _add_keyword(hdrlist, name, val):
@@ -140,11 +142,11 @@ def _add_keyword(hdrlist, name, val):
 
 
 def _try_col(hdu, name, dtype=SherpaFloat, fix_type=False):
-    if name not in hdu.columns.names:
+    try:
+        col = hdu.data.field(name)
+    except KeyError:
         return None
 
-    col = hdu.data.field(name)
-
     if isinstance(col, _VLF):
         col = numpy.concatenate([numpy.asarray(row) for row in col])
     else:
@@ -157,10 +159,10 @@ def _try_col(hdu, name, dtype=SherpaFloat, fix_type=False):
 
 
 def _try_tbl_col(hdu, name, dtype=SherpaFloat, fix_type=False):
-    if name not in hdu.columns.names:
-        return (None,)
-
-    col = hdu.data.field(name)
+    try:
+        col = hdu.data.field(name)
+    except KeyError:
+        return (None, )
 
     if isinstance(col, _VLF):
         col = numpy.concatenate([numpy.asarray(row) for row in col])
@@ -174,11 +176,11 @@ def _try_tbl_col(hdu, name, dtype=SherpaFloat, fix_type=False):
 
 
 def _try_vec(hdu, name, size=2, dtype=SherpaFloat, fix_type=False):
-    if name not in hdu.columns.names:
+    try:
+        col = hdu.data.field(name)
+    except KeyError:
         return numpy.array([None] * size)
 
-    col = hdu.data.field(name)
-
     if isinstance(col, _VLF):
         col = numpy.concatenate([numpy.asarray(row) for row in col])
     else:
@@ -236,12 +238,15 @@ def _try_vec_or_key(hdu, name, size, dtype=SherpaFloat, fix_type=False):
 # effort to emulate with the astropy backend.
 
 def _infer_and_check_filename(filename):
-    fname = filename  # keep filename unchanged
-    if not os.path.exists(fname):
-        fname += '.gz'  # try to find a gzipped version, following CXC/Crates conventions.
-        if not os.path.exists(fname):  # fail fast
-            raise IOErr('filenotfound', filename)  # error message reports the original filename requested
-    return fname
+    if os.path.exists(filename):
+        return filename
+
+    gzname = f"{filename}.gz"
+    if os.path.exists(gzname):
+        return gzname
+
+    # error message reports the original filename requested
+    raise IOErr('filenotfound', filename)
 
 
 def is_binary_file(filename):
@@ -306,7 +311,7 @@ def read_table_blocks(arg, make_copy=False):
     if isinstance(arg, fits.HDUList):
         filename = arg[0]._file.name
         hdus = arg
-    elif isinstance(arg, basestring) and is_binary_file(arg):
+    elif isinstance(arg, str) and is_binary_file(arg):
         filename = arg
         hdus = open_fits(arg)
     else:
@@ -358,7 +363,7 @@ def _get_file_contents(arg, exptype="PrimaryHDU", nobinary=False):
     file is a binary file (via the is_binary_file routine).
     """
 
-    if isinstance(arg, basestring) and (not nobinary or is_binary_file(arg)):
+    if isinstance(arg, str) and (not nobinary or is_binary_file(arg)):
         tbl = open_fits(arg)
         filename = arg
     elif isinstance(arg, fits.HDUList) and len(arg) > 0 and \
@@ -366,7 +371,7 @@ def _get_file_contents(arg, exptype="PrimaryHDU", nobinary=False):
         tbl = arg
         filename = tbl[0]._file.name
     else:
-        msg = "a binary FITS table or a {} list".format(exptype)
+        msg = f"a binary FITS table or a {exptype} list"
         raise IOErr('badfile', arg, msg)
 
     return (tbl, filename)
@@ -686,7 +691,7 @@ def get_rmf_data(arg, make_copy=False):
 
         # Beginning of non-Chandra RMF support
         fchan_col = list(hdu.columns.names).index('F_CHAN') + 1
-        tlmin = _try_key(hdu, 'TLMIN' + str(fchan_col), True, SherpaUInt)
+        tlmin = _try_key(hdu, f"TLMIN{fchan_col}", True, SherpaUInt)
 
         if tlmin is not None:
             data['offset'] = tlmin
@@ -694,7 +699,7 @@ def get_rmf_data(arg, make_copy=False):
             # QUS: should this actually be an error, rather than just
             #      something that is logged to screen?
             error("Failed to locate TLMIN keyword for F_CHAN" +
-                  " column in RMF file '%s'; " % filename +
+                  f" column in RMF file '{filename}'; " +
                   'Update the offset value in the RMF data set to' +
                   ' appropriate TLMIN value prior to fitting')
 
@@ -705,7 +710,7 @@ def get_rmf_data(arg, make_copy=False):
 
             # Beginning of non-Chandra RMF support
             chan_col = list(hdu.columns.names).index('CHANNEL') + 1
-            tlmin = _try_key(hdu, 'TLMIN' + str(chan_col), True, SherpaUInt)
+            tlmin = _try_key(hdu, f"TLMIN{chan_col}", True, SherpaUInt)
             if tlmin is not None:
                 data['offset'] = tlmin
 
@@ -720,7 +725,7 @@ def get_rmf_data(arg, make_copy=False):
     # Note that crates uses the sherpa.astro.utils.resp_init routine,
     # but it's not clear why, so it is not used here for now.
     #
-    good = (data['n_grp'] > 0)
+    good = data['n_grp'] > 0
     data['matrix'] = data['matrix'][good]
 
     if isinstance(data['matrix'], _VLF):
@@ -790,7 +795,7 @@ def get_rmf_data(arg, make_copy=False):
     else:
         if len(data['n_grp']) == len(data['f_chan']):
             # filter out groups with zeroes.
-            good = (data['n_grp'] > 0)
+            good = data['n_grp'] > 0
             data['f_chan'] = data['f_chan'][good]
             data['n_chan'] = data['n_chan'][good]
 
@@ -842,7 +847,7 @@ def get_pha_data(arg, make_copy=False, use_background=False):
 
             # Make sure channel numbers not indices
             chan = list(hdu.columns.names).index('CHANNEL') + 1
-            tlmin = _try_key(hdu, 'TLMIN' + str(chan), True, SherpaUInt)
+            tlmin = _try_key(hdu, f"TLMIN{chan}", True, SherpaUInt)
             if int(data['channel'][0]) == 0 or tlmin == 0:
                 data['channel'] = data['channel'] + 1
 
@@ -897,7 +902,7 @@ def get_pha_data(arg, make_copy=False, use_background=False):
 
             # Make sure channel numbers not indices
             chan = list(hdu.columns.names).index('CHANNEL') + 1
-            tlmin = _try_key(hdu, 'TLMIN' + str(chan), True, SherpaUInt)
+            tlmin = _try_key(hdu, f"TLMIN{chan}", True, SherpaUInt)
 
             for ii in range(num):
                 if int(channel[ii][0]) == 0:
@@ -1006,11 +1011,10 @@ def _create_columns(col_names, data):
 def set_table_data(filename, data, col_names, hdr=None, hdrnames=None,
                    ascii=False, clobber=False, packup=False):
 
-    if not packup and os.path.isfile(filename) and not clobber:
+    if not packup and not clobber and os.path.isfile(filename):
         raise IOErr("filefound", filename)
 
     col_names = list(col_names)
-    col_names.remove("name")
 
     # The code used to create a header containing the
     # exposure, backscal, and areascal keywords, but this has
@@ -1109,7 +1113,7 @@ def set_image_data(filename, data, header, ascii=False, clobber=False,
         if data['eqpos'] is not None:
             # Simply the inverse of read transformations in get_image_data
             cdeltw = cdeltw * cdeltp
-            crpixw = ((crpixw - crvalp) / cdeltp + crpixp)
+            crpixw = (crpixw - crvalp) / cdeltp + crpixp
 
     if data['eqpos'] is not None:
         _add_keyword(hdrlist, 'MTYPE2', 'EQPOS   ')
@@ -1152,7 +1156,7 @@ def set_arrays(filename, args, fields=None, ascii=True, clobber=False):
             raise IOErr('arraysnoteq')
 
     if not ascii and fields is None:
-        fields = ['col%i' % (ii + 1) for ii in range(len(args))]
+        fields = [f'col{ii + 1}' for ii in range(len(args))]
 
     if fields is not None and len(args) != len(fields):
         raise IOErr("wrongnumcols", len(args), len(fields))
