@@ -1,5 +1,6 @@
 #
-#  Copyright (C) 2016, 2018, 2020, 2021  Smithsonian Astrophysical Observatory
+#  Copyright (C) 2016, 2018, 2020, 2021, 2022
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -17,6 +18,7 @@
 #  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
+import importlib
 import logging
 import os
 import sys
@@ -70,7 +72,7 @@ def run(verbosity=0, require_failure=False, fits=None, xspec=False, ds9=False):
         @unittest.skipIf(not fits, reason="FITS dependency not requested")
         def test_fits_dependency(self):
             try:
-                __import__(fits, globals(), locals())
+                importlib.import_module(fits)
             except ImportError:
                 missing_requirements.append(_import_error(fits, "fits"))
                 raise
@@ -130,6 +132,7 @@ class SmokeTest(unittest.TestCase):
         self.x = np.asarray([1, 2, 3])
         self.x2 = self.x + 1
         self.y = np.asarray([1, 2, 3])
+        ui.clean()
 
     def tearDown(self):
         ui.clean()
@@ -145,9 +148,8 @@ class SmokeTest(unittest.TestCase):
         ui.set_method("levmar")
         ui.fit()
         model = ui.get_model_component("p")
-        expected = [0, 1]
-        observed = [model.c0.val, model.c1.val]
-        assert_almost_equal(observed, expected)
+        assert_almost_equal(model.c0.val, 0)
+        assert_almost_equal(model.c1.val, 1)
 
     # Using skipIf directly as we need these tests to run when there is no pytest installed, so we can't use
     # the decorators in sherpa.utils.testing
