@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2008, 2015, 2016, 2017, 2019, 2020, 2021
+#  Copyright (C) 2008, 2015, 2016, 2017, 2019, 2020, 2021, 2022
 #  Smithsonian Astrophysical Observatory
 #
 #
@@ -83,9 +83,9 @@ def _check(array):
 
     if hasattr(array, "shape"):
         if len(array.shape) != 1:
-            raise TypeError("Data arrays should be 1-dimensional. Did you call 'flatten()' on {}?)".format(array))
+            raise TypeError(f"Data arrays should be 1-dimensional. Did you call 'flatten()' on {array}?)")
     else:
-        warnings.warn("Converting array {} to numpy array.".format(array))
+        warnings.warn(f"Converting array {array} to numpy array.")
         array = numpy.asanyarray(array)
         return _check(array)
     return array
@@ -93,28 +93,28 @@ def _check(array):
 
 def _check_nomask(array):
     if hasattr(array, 'mask'):
-        warnings.warn('Input array {} has a mask attribute. Because masks are supported for dependent variables only the mask attribute of the independent array is ignored and values `behind the mask` are used.'.format(array))
+        warnings.warn(f'Input array {array} has a mask attribute. Because masks are supported for dependent variables only the mask attribute of the independent array is ignored and values `behind the mask` are used.')
     return array
 
 
 def _check_dep(array):
     if not hasattr(array, 'mask'):
         return _check(array), True
-    else:
-        # We know the mask convention is opposite to sherpa
-        if isinstance(array, numpy.ma.MaskedArray):
-            return _check(array), ~array.mask
-        # We don't know what the mask convention is
-        else:
-            warnings.warn('Format of mask for array {} not supported thus the mask is is ignored and values `behind the mask` are used. Set .mask attribute manually or use "set_filter" function.'.format(array))
-            return _check(array), True
+
+    # We know the mask convention is opposite to sherpa
+    if isinstance(array, numpy.ma.MaskedArray):
+        return _check(array), ~array.mask
+
+    # We don't know what the mask convention is
+    warnings.warn(f'Format of mask for array {array} not supported thus the mask is is ignored and values `behind the mask` are used. Set .mask attribute manually or use "set_filter" function.')
+    return _check(array), True
 
 
 def _check_err(array, masktemplate):
     '''Accept array without mask or with a mask that matches the template'''
     if hasattr(array, 'mask') and \
        (not hasattr(masktemplate, 'mask') or not numpy.all(array.mask == masktemplate.mask)):
-        warnings.warn('The mask of {} differs from the mask of the dependent array, only the mask of the dependent array is used in Sherpa.'.format(array))
+        warnings.warn(f'The mask of {array} differs from the mask of the dependent array, only the mask of the dependent array is used in Sherpa.')
     return array
 
 
@@ -134,7 +134,7 @@ class DataSpace1D(EvaluationSpace1D):
             the x axis of this data space
         """
         self.filter = filter
-        EvaluationSpace1D.__init__(self, _check_nomask(x))
+        super().__init__(_check_nomask(x))
 
     def get(self, filter=False):
         """
@@ -200,7 +200,7 @@ class IntegratedDataSpace1D(EvaluationSpace1D):
             the higher bounds array of this data space
         """
         self.filter = filter
-        EvaluationSpace1D.__init__(self, _check_nomask(xlo), _check_nomask(xhi))
+        super().__init__(_check_nomask(xlo), _check_nomask(xhi))
 
     def get(self, filter=False):
         """
@@ -612,43 +612,60 @@ class BaseData(metaclass=ABCMeta):
 
 # DATA-NOTE: ND Data cannot be plotted
 class Data(NoNewAttributesAfterInit, BaseData):
+    """Generic, N-Dimensional data sets.
+
+    A data class is the collection of a data space and a number of
+    data arrays for the dependent variable and associated errors.
+
+    Parameters
+    ----------
+    name : string
+        name of this dataset
+    indep: tuple of array_like
+        the tuple of independent arrays.
+    y : array_like
+        The values of the dependent observable. If this is a numpy
+        masked array, the mask will used to initialize a mask.
+    staterror : array_like
+        the statistical error associated with the data
+    syserror : array_like
+        the systematic error associated with the data
+
+    Notes
+    -----
+
+    This class can be extended by classes definining data sets of
+    specific dimensionality. Extending classes should override the
+    `_init_data_space` method.
+
+    This class provides most of the infrastructure for extending
+    classes for free.
+
+    Data classes contain a ``mask`` attribute, which can be used
+    ignore certain values in the array when fitting or plotting that
+    data. The convention in Sherpa is that ``True`` marks a values as
+    *valid* and ``False`` as *invalid* (note that this is opposite to
+    the numpy convention). When a `Data` instance is initialized with
+    a dependent array that has a ``mask`` attribute (e.g. numpy masked
+    array), it will attempt to convert that mask to the Sherpa
+    convention and raise a warning otherwise. In any case, the user
+    can set ``data.mask`` after initialization if that conversion does
+    not yield the expected result.
+
     """
-    Data class for generic, N-Dimensional data sets, where N depends on the number of independent axes passed during
-    initialization.
 
-    A data class is the collection of a data space and a number of data arrays for the dependent variable and
-    associated errors.
-
-    This class can be extended by classes definining data sets of specific dimensionality. Extending classes should
-    override the `_init_data_space` method.
-
-    This class provides most of the infrastructure for extending classes for free.
-
-    Data classes contain a ``mask`` attribute, which can be used ignore certain values in the array
-    when fitting or plotting that data. The convention in Sherpa is that ``True`` marks a values as
-    *valid* and ``False`` as *invalid* (note that this is opposite to the numpy convention). When a `Data`
-    instance is initialized with a dependent array that has a ``mask`` attribute (e.g. numpy masked array),
-    it will attempt to convert that mask to the Sherpa convention and raise a warning otherwise. In any case,
-    the user can set ``data.mask`` after initialization if that conversion does not yield the expected result.
-    """
     _fields = ("name", "indep", "dep", "staterror", "syserror")
+    """The main data values stored by the object (as a tuple).
+
+    This is used to identify the column data - that is values that
+    can be written out in tabular form - other than the "name"
+    field. Other fields are listed in _extra_fields.
+    """
+
+    _extra_fields = ()
+    """Any extra fields that should be displayed by str(object)."""
 
     def __init__(self, name, indep, y, staterror=None, syserror=None):
-        """
-        Parameters
-        ----------
-        name : basestring
-            name of this dataset
-        indep: tuple of array_like
-            the tuple of independent arrays.
-        y : array_like
-            The values of the dependent observable. If this is a numpy masked array,
-            the mask will used to initialize a mask.
-        staterror : array_like
-            the statistical error associated with the data
-        syserror : array_like
-            the systematic error associated with the data
-        """
         self.name = name
         self._data_space = self._init_data_space(Filter(), *indep)
         self.y, self.mask = _check_dep(y)
@@ -786,7 +803,7 @@ class Data(NoNewAttributesAfterInit, BaseData):
 
     def set_dep(self, val):
         """
-        Set the dependent variable values"
+        Set the dependent variable values.
 
         Parameters
         ----------
@@ -984,17 +1001,17 @@ class Data(NoNewAttributesAfterInit, BaseData):
     def __str__(self):
         """
         Return a listing of the attributes listed in self._fields and,
-        if present, self._extra_fields.
+        if set, self._extra_fields.
         """
 
-        fields = self._fields + getattr(self, '_extra_fields', ())
-        fdict = dict(zip(fields, [getattr(self, f) for f in fields]))
+        fields = self._fields + self._extra_fields
+        fdict = {f: getattr(self, f) for f in fields}
         return print_fields(fields, fdict)
 
     def __repr__(self):
-        r = '<%s data set instance' % type(self).__name__
+        r = f'<{type(self).__name__} data set instance'
         if hasattr(self, 'name'):
-            r += " '%s'" % self.name
+            r += f" '{self.name}'"
         r += '>'
         return r
 
@@ -1042,7 +1059,7 @@ class DataSimulFit(NoNewAttributesAfterInit):
         self.name = name
         self.datasets = tuple(datasets)
         self.numcores = numcores
-        NoNewAttributesAfterInit.__init__(self)
+        super().__init__()
 
     def eval_model_to_fit(self, modelfuncs):
         if self.numcores == 1:
@@ -1113,7 +1130,7 @@ class Data1D(Data):
     _fields = ("name", "x", "y", "staterror", "syserror")
 
     def __init__(self, name, x, y, staterror=None, syserror=None):
-        Data.__init__(self, name, (x, ), y, staterror, syserror)
+        super().__init__(name, (x, ), y, staterror, syserror)
 
     def _repr_html_(self):
         """Return a HTML (string) representation of the data
@@ -1373,7 +1390,7 @@ class Data1DAsymmetricErrs(Data1D):
     def __init__(self, name, x, y, elo, ehi, staterror=None, syserror=None):
         self.elo = elo
         self.ehi = ehi
-        Data1D.__init__(self, name, x, y, staterror=staterror, syserror=syserror)
+        super().__init__(name, x, y, staterror=staterror, syserror=syserror)
 
     def get_yerr(self, filter=False, staterrfunc=None):
         return self.elo, self.ehi
@@ -1553,9 +1570,17 @@ class Data1DInt(Data1D):
 class Data2D(Data):
     _fields = ("name", "x0", "x1", "y", "shape", "staterror", "syserror")
 
+    # Why should we add shape to extra-fields instead? See #1359 to
+    # fix #47.
+    #
+    # It would change the notebook output slightly so we don't change
+    # for now.
+    #
+    # _extra_fields = ("shape", )
+
     def __init__(self, name, x0, x1, y, shape=None, staterror=None, syserror=None):
         self.shape = shape
-        Data.__init__(self, name, (x0, x1), y, staterror, syserror)
+        super().__init__(name, (x0, x1), y, staterror, syserror)
 
     def _repr_html_(self):
         """Return a HTML (string) representation of the data
@@ -1606,13 +1631,35 @@ class Data2D(Data):
         return ''
 
     def get_max_pos(self, dep=None):
+        """Return the coordinates of the maximum value.
+
+        Parameters
+        ----------
+        dep : ndarray or None, optional
+            The data to search and it must match the current data
+            filter. If not given then the dependent axis is used.
+
+        Returns
+        -------
+        coords : pair or list of pairs
+            The coordinates of the maximum location. The data values
+            match the values returned by `get_x0` and `get_x1`. If
+            there is only one maximum pixel then a pair is returned
+            otherwise a list of pairs is returned.
+
+        See Also
+        --------
+        get_dep, get_x0, get_x1
+
+        """
         if dep is None:
             dep = self.get_dep(True)
         x0 = self.get_x0(True)
         x1 = self.get_x1(True)
 
+        # TODO: Should be able to just use numpy.argmax
         pos = numpy.asarray(numpy.where(dep == dep.max())).squeeze()
-        if pos.ndim == 0:  # DATA-NOTE: Could this ever be False?!
+        if pos.ndim == 0:
             pos = int(pos)
             return x0[pos], x1[pos]
 
@@ -1622,21 +1669,39 @@ class Data2D(Data):
     # Also, we do not filter, as imager needs M x N (or
     # L x M x N) array
     def get_img(self, yfunc=None):
+        """Return the dependent axis as a 2D array.
+
+        The data is not filtered.
+
+        Parameters
+        ----------
+        yfunc : sherpa.models.model.Model instance or None, optional
+            If set then it is a model that is evaluated on the data
+            grid and returned along with the dependent axis.
+
+        Returns
+        -------
+        img : ndarray or (ndarray, ndarray)
+            The data as a 2D array or a pair of 2D arrays when yfunc
+            is set.
+
+        """
+
         self._check_shape()
         y_img = self.get_y(False, yfunc)
-        if yfunc is not None:
-            y_img = (y_img[0].reshape(*self.shape),
-                     y_img[1].reshape(*self.shape))
-        else:
-            y_img = y_img.reshape(*self.shape)
-        return y_img
+        if yfunc is None:
+            return y_img.reshape(*self.shape)
+
+        return (y_img[0].reshape(*self.shape),
+                y_img[1].reshape(*self.shape))
 
     def get_imgerr(self):
         self._check_shape()
         err = self.get_error()
-        if err is not None:
-            err = err.reshape(*self.shape)
-        return err
+        if err is None:
+            return None
+
+        return err.reshape(*self.shape)
 
     def to_contour(self, yfunc=None):
         return (self.get_x0(True),
@@ -1672,7 +1737,8 @@ class Data2DInt(Data2D):
     """
     2-D integrated data set
     """
-    _fields = ("name", "x0lo", "x1lo", "x0hi", "x1hi", "y", "shape", "staterror", "syserror")
+    _fields = ("name", "x0lo", "x1lo", "x0hi", "x1hi", "y", "staterror", "syserror")
+    _extra_fields = ("shape", )
 
     def __init__(self, name, x0lo, x1lo, x0hi, x1hi, y, shape=None, staterror=None, syserror=None):
         self.shape = shape
@@ -1693,6 +1759,34 @@ class Data2DInt(Data2D):
         Data.notice(self, (None, None, x0lo, x1lo), (x0hi, x1hi, None, None),
                     ignore=ignore, integrated=True)
 
+    @property
+    def x0lo(self):
+        """
+        Property kept for compatibility
+        """
+        return self._data_space.x0lo
+
+    @property
+    def x0hi(self):
+        """
+        Property kept for compatibility
+        """
+        return self._data_space.x0hi
+
+    @property
+    def x1lo(self):
+        """
+        Property kept for compatibility
+        """
+        return self._data_space.x1lo
+
+    @property
+    def x1hi(self):
+        """
+        Property kept for compatibility
+        """
+        return self._data_space.x1hi
+
 
 # Notebook representations
 #
@@ -1702,16 +1796,16 @@ def html_data1d(data):
     If have matplotlib then plot the data, otherwise summarize it.
     """
 
-    from sherpa.plot import DataPlot, backend
+    from sherpa import plot
 
     dtype = type(data).__name__
 
-    plotter = DataPlot()
+    plotter = plot.DataPlot()
     plotter.prepare(data)
 
-    summary = '{} Plot'.format(dtype)
+    summary = f'{dtype} Plot'
     try:
-        out = backend.as_html_plot(plotter, summary)
+        out = plot.backend.as_html_plot(plotter, summary)
     except AttributeError:
         out = None
 
@@ -1730,9 +1824,11 @@ def html_data1d(data):
     #
     fexpr = data.get_filter_expr()
     nbins = data.get_dep(filter=True).size
-    meta.append(('Using', '{} with {} bins'.format(fexpr, nbins)))
+    meta.append(('Using', f'{fexpr} with {nbins} bins'))
 
-    # Rely on the _fields ordering, ending at staterror
+    # Rely on the _fields ordering, ending at staterror. This
+    # ignores _extra_fields.
+    #
     for f in data._fields[1:]:
         if f == 'staterror':
             break
@@ -1745,7 +1841,7 @@ def html_data1d(data):
     if data.syserror is not None:
         meta.append(('Systematic error', data.syserror))
 
-    ls = [formatting.html_section(meta, summary=dtype + ' Summary',
+    ls = [formatting.html_section(meta, summary=f'{dtype} Summary',
                                   open_block=True)]
     return formatting.html_from_sections(data, ls)
 
@@ -1756,16 +1852,16 @@ def html_data1dint(data):
     If have matplotlib then plot the data, otherwise summarize it.
     """
 
-    from sherpa.plot import DataHistogramPlot, backend
+    from sherpa import plot
 
     dtype = type(data).__name__
 
-    plotter = DataHistogramPlot()
+    plotter = plot.DataHistogramPlot()
     plotter.prepare(data)
 
-    summary = '{} Plot'.format(dtype)
+    summary = f'{dtype} Plot'
     try:
-        out = backend.as_html_plot(plotter, summary)
+        out = plot.backend.as_html_plot(plotter, summary)
     except AttributeError:
         out = None
 
@@ -1784,7 +1880,7 @@ def html_data1dint(data):
     #
     fexpr = data.get_filter_expr()
     nbins = data.get_dep(filter=True).size
-    meta.append(('Using', '{} with {} bins'.format(fexpr, nbins)))
+    meta.append(('Using', f'{fexpr} with {nbins} bins'))
 
     # Rely on the _fields ordering, ending at staterror
     for f in data._fields[1:]:
@@ -1799,7 +1895,7 @@ def html_data1dint(data):
     if data.syserror is not None:
         meta.append(('Systematic error', data.syserror))
 
-    ls = [formatting.html_section(meta, summary=dtype + ' Summary',
+    ls = [formatting.html_section(meta, summary=f'{dtype} Summary',
                                   open_block=True)]
     return formatting.html_from_sections(data, ls)
 
@@ -1849,6 +1945,6 @@ def html_data2d(data):
     if data.syserror is not None:
         meta.append(('Systematic error', data.syserror))
 
-    ls = [formatting.html_section(meta, summary=dtype + ' Summary',
+    ls = [formatting.html_section(meta, summary=f'{dtype} Summary',
                                   open_block=True)]
     return formatting.html_from_sections(data, ls)
