@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2009, 2015, 2016, 2018, 2019, 2020, 2021
+#  Copyright (C) 2009, 2015, 2016, 2018, 2019, 2020, 2021, 2022
 #  Smithsonian Astrophysical Observatory
 #
 #
@@ -566,7 +566,7 @@ class IterFit(NoNewAttributesAfterInit):
         # Options to send to iterative fitting method
         self.itermethod_opts = itermethod_opts
         self.iterate = False
-        self.funcs = {'primini': self.primini, 'sigmarej': self.sigmarej}
+        self.funcs = {'sigmarej': self.sigmarej}
         self.current_func = None
         if itermethod_opts['name'] != 'none':
             self.current_func = self.funcs[itermethod_opts['name']]
@@ -622,152 +622,6 @@ class IterFit(NoNewAttributesAfterInit):
 
         return cb
 
-    def primini(self, statfunc, pars, parmins, parmaxes, statargs=(),
-                statkwargs=None):
-        r"""An iterative scheme, where the variance is computed from
-        the model amplitudes.
-
-        This is a chi-square statistic where the variance is computed
-        from model amplitudes derived in the previous iteration of the
-        fit. This 'Iterative Weighting' [1]_ attempts to remove
-        biased estimates of model parameters which is inherent in
-        chi-square statistics [2]_.
-
-        Raises
-        ------
-        `sherpa.utils.err.FitErr`
-            This exception is raised if the statistic is not supported.
-            This method can only be used with Chi-Square statistics
-            with errors.
-
-        Notes
-        -----
-        The following keys are looked for in the `itermethod_opts`
-        dictionary:
-
-        ========  =========  ===========
-        Key       Type       Description
-        ========  =========  ===========
-        tol       number     The tolerance used when comparing the statistic
-                             values.
-        maxiters  int        The maximum number of iterations.
-        ========  =========  ===========
-
-        The variance in bin i for iteration j is estimated to be:
-
-        .. math::
-
-           \sigma(j)_i^2 = S(t_s(j-1))_i + (A_s/A_b)^2 B(t_b(j-1))_i
-
-        where the subscript i indicates the bin number, j is the
-        number of iterations that have been carried out in the
-        fitting process, S is the model-predicted amplitude of
-        the source in the bin, B is the background (off-source)
-        model amplitude, A_s and A_b are the source and background
-        areas (or scaling factors), and t_s and t_b are the set
-        of source and background model parameter values derived in
-        the previous iteration (indicated by the j-1 term). The
-        variances are set to an array of ones on the first iteration.
-
-        In addition to reducing parameter estimate bias, this
-        statistic can be used even when the number of counts in each
-        bin is small (< 5), although the user should proceed with
-        caution.
-
-        References
-        ----------
-
-        .. [1] "Multiparameter linear least-squares fitting to Poisson
-               data one count at a time", Wheaton et al. 1995, ApJ 438,
-               322
-               http://adsabs.harvard.edu/abs/1995ApJ...438..322W
-
-        .. [2] "Bias-Free Parameter Estimation with Few Counts, by
-               Iterative Chi-Squared Minimization", Kearns, Primini, &
-               Alexander, 1995, ADASS IV, 331
-               http://adsabs.harvard.edu/abs/1995ASPC...77..331K
-
-        """
-        if statkwargs is None:
-            statkwargs = {}
-        # Primini's method can only be used with chi-squared;
-        # raise exception if it is attempted with least-squares,
-        # or maximum likelihood
-        if (isinstance(self.stat, Chi2) and
-                type(self.stat) is not LeastSq):
-            pass
-        else:
-            raise FitErr('needchi2', 'Primini\'s')
-        # Get tolerance, max number of iterations from the
-        # dictionary for Primini's method
-        tol = self.itermethod_opts['tol']
-        if type(tol) != int and type(tol) != float:
-            raise SherpaErr(
-                "'tol' value for Primini's method must be a number")
-        maxiters = self.itermethod_opts['maxiters']
-        if type(maxiters) != int:
-            raise SherpaErr(
-                "'maxiters' value for Primini's method must be an integer")
-
-        # Store original statistical errors for all data sets.
-        # Then, set all statistical errors equal to one, to
-        # prepare for Primini's method.
-        staterror_original = []
-
-        for d in self.data.datasets:
-            st = d.get_staterror(filter=False)
-            staterror_original.append(st)
-            d.staterror = ones_like(st)
-
-        # Keep record of current and previous statistics;
-        # when these are within some tolerace, Primini's method
-        # is done.
-        previous_stat = np.finfo(np.float32).max
-        current_stat = statfunc(pars)[0]
-        nfev = 0
-        iters = 0
-
-        # This is Primini's method.  The essence of the method
-        # is to fit; then call the statistic's staterror function
-        # on model values (calculated with current parameter values);
-        # then call fit *again*.  Do this until the final fit
-        # statistic for the previous and current calls to fit
-        # agree to within tolerance.
-        final_fit_results = None
-        try:
-            while (iters < maxiters and
-                   sao_fcmp(previous_stat, current_stat, tol) != 0):
-                final_fit_results = self.method.fit(statfunc,
-                                                    self.model.thawedpars,
-                                                    parmins, parmaxes,
-                                                    statargs, statkwargs)
-                previous_stat = current_stat
-                current_stat = final_fit_results[2]
-                nfev += final_fit_results[4].get('nfev')
-                iters += 1
-
-                # Call stat.staterror with *model*values*, not data
-                # Model values calculated using best-fit parameter
-                # values from the just-completed call to the fit
-                # function.
-                model_iterator = iter(self.model())
-                for d in self.data.datasets:
-                    d.staterror = self.stat.calc_staterror(
-                        d.eval_model(next(model_iterator)))
-
-            # Final number of function evaluations is the sum
-            # of the numbers of function evaluations from all calls
-            # to the fit function.
-            final_fit_results[4]['nfev'] = nfev
-        finally:
-            # Clean up *no*matter*what* -- we must always
-            # restore original statistical errors.
-            staterror_original.reverse()
-            for d in self.data.datasets:
-                d.staterror = staterror_original.pop()
-
-        # Return results from Primini's iterative fitting method
-        return final_fit_results
 
     def sigmarej(self, statfunc, pars, parmins, parmaxes, statargs=(),
                  statkwargs=None, cache=True):
