@@ -306,6 +306,13 @@ def test_Branin(opt, npar=2):
 def test_Colville(opt, npar=4):
     tst_opt(opt, _tstoptfct.Colville, npar)
 
+def test_minim_no_reflect(reltol=1.0e-3, abstol=1.0e-3):
+     fct = _tstoptfct.Colville
+     wrong_fval = 174.28569111739617
+     x0, xmin, xmax, fmin = init( fct.__name__, 4)
+     status, x, fval, msg, xtra = minim(fct, x0, xmin, xmax, reflect=False)
+     assert fval == pytest.approx(wrong_fval, rel=reltol, abs=abstol)
+     assert fmin != wrong_fval
 
 # @pytest.mark.parametrize("opt", [pytest.param(minim, marks=pytest.mark.xfail),
 #                                  pytest.param(montecarlo,
