@@ -31,7 +31,7 @@ from sherpa.plot.utils import histogram_line
 
 
 __all__ = ('clear_window', 'point', 'plot', 'histo', 'contour', 'set_subplot',
-           'init', 'get_split_plot_defaults', 'get_plot_defaults', 'begin',
+           'get_split_plot_defaults', 'get_plot_defaults', 'begin',
            'end', 'get_data_plot_defaults', 'get_model_plot_defaults',
            'exceptions', 'get_fit_plot_defaults', 'get_resid_plot_defaults',
            'get_ratio_plot_defaults', 'get_contour_defaults',
@@ -42,7 +42,8 @@ __all__ = ('clear_window', 'point', 'plot', 'histo', 'contour', 'set_subplot',
            'get_model_histo_defaults', 'get_histo_defaults',
            'get_component_plot_defaults', 'get_component_histo_defaults',
            'vline', 'hline', 'get_scatter_plot_defaults',
-           'get_cdf_plot_defaults', 'get_latex_for_string', 'name')
+           'get_cdf_plot_defaults', 'get_latex_for_string', 'name',
+           'initialize_plot', 'select_plot')
 
 
 # Name of the backend
@@ -51,10 +52,6 @@ name = 'pylab'
 logger = logging.getLogger(__name__)
 
 
-def init():
-    pass
-
-
 def begin():
     pass
 
@@ -69,9 +66,6 @@ def exceptions():
     pass
 
 
-_errorbar_defaults = get_keyword_defaults(plt.errorbar)
-
-
 def clear_window():
     plt.clf()
 
@@ -211,9 +205,9 @@ def histo(xlo, xhi, y, yerr=None, title=None, xlabel=None, ylabel=None,
           overplot=False, clearwindow=True,
           xerrorbars=False,
           yerrorbars=False,
-          ecolor=_errorbar_defaults['ecolor'],
-          capsize=_errorbar_defaults['capsize'],
-          barsabove=_errorbar_defaults['barsabove'],
+          ecolor=None,
+          capsize=None,
+          barsabove=False,
           xlog=False,
           ylog=False,
           linestyle='solid',
@@ -364,9 +358,9 @@ def plot(x, y, yerr=None, xerr=None, title=None, xlabel=None, ylabel=None,
          overplot=False, clearwindow=True,
          xerrorbars=False,
          yerrorbars=False,
-         ecolor=_errorbar_defaults['ecolor'],
-         capsize=_errorbar_defaults['capsize'],
-         barsabove=_errorbar_defaults['barsabove'],
+         ecolor=None,
+         capsize=None,
+         barsabove=False,
          xlog=False,
          ylog=False,
          linestyle='solid',
@@ -835,3 +829,43 @@ as_html_fit = as_html_plot
 as_html_fitcontour = as_html_contour
 as_html_contour1d = as_html_plot
 as_html_contour2d = as_html_contour
+
+# Needed for datastack plotting wrapper
+
+
+def initialize_plot(dataset, ids):
+    """Create the plot window or figure for the given dataset.
+
+    Parameters
+    ----------
+    dataset : str or int
+       The dataset.
+    ids : array_like
+       The identifier array from the DataStack object.
+
+    See Also
+    --------
+    select_plot
+
+    """
+    plt.figure(ids.index(dataset['id']) + 1)
+
+
+def select_plot(dataset, ids):
+    """Select the plot window or figure for the given dataset.
+
+    The plot for this dataset is assumed to have been created.
+
+    Parameters
+    ----------
+    dataset : str or int
+       The dataset.
+    ids : array_like
+       The identifier array from the DataStack object.
+
+    See Also
+    --------
+    initialize_plot
+
+    """
+    plt.figure(ids.index(dataset['id']) + 1)
