@@ -1,5 +1,6 @@
 #
-# Copyright (C) 2020, 2021  Smithsonian Astrophysical Observatory
+# Copyright (C) 2020, 2021
+# Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -34,16 +35,20 @@ from sherpa.models.basic import Const1D, Gauss2D, Polynom1D
 from sherpa.stats import Chi2
 
 
+def plot_backend_is(name):
+    """Check we have the specified plot backend"""
+    return plot.backend.name == name
+
+
 def check_empty(r, summary, nsummary=0):
     """Is this an 'empty' response?"""
 
-    if plot.backend.name == 'pylab':
+    if plot_backend_is("pylab"):
         assert r is None
         return
 
     assert r is not None
-
-    assert '<summary>{} ({})</summary>'.format(summary, nsummary) in r
+    assert f"<summary>{summary} ({nsummary})</summary>" in r
 
 
 def check_full(r, summary, label, title, nsummary=0):
@@ -51,14 +56,14 @@ def check_full(r, summary, label, title, nsummary=0):
 
     assert r is not None
 
-    if plot.backend.name == 'pylab':
-        assert '<summary>{}</summary>'.format(summary) in r
-        assert '<svg ' in r
+    if plot_backend_is("pylab"):
+        assert f"<summary>{summary}</summary>" in r
+        assert "<svg " in r
         return
 
-    assert '<summary>{} ({})</summary>'.format(summary, nsummary) in r
-    assert '<div class="dataval">{}</div>'.format(label) in r
-    assert '<div class="dataval">{}</div>'.format(title) in r
+    assert f"<summary>{summary} ({nsummary})</summary>" in r
+    assert f'<div class="dataval">{label}</div>' in r
+    assert f'<div class="dataval">{title}</div>' in r
 
 
 def test_histogram(override_plot_backend):
@@ -205,13 +210,13 @@ def test_fit(override_plot_backend):
     # different to previous checks
     assert r is not None
 
-    if plot.backend.name == 'pylab':
-        assert '<summary>FitPlot</summary>' in r
-        assert '<svg ' in r
+    if plot_backend_is("pylab"):
+        assert "<summary>FitPlot</summary>" in r
+        assert "<svg " in r
         return
 
-    assert '<summary>DataPlot (' in r
-    assert '<summary>ModelPlot (' in r
+    assert "<summary>DataPlot (" in r
+    assert "<summary>ModelPlot (" in r
     assert '<div class="dataval">n n</div>' in r
     assert '<div class="dataval">Model</div>' in r
 
@@ -244,13 +249,13 @@ def test_fitcontour(override_plot_backend):
     # different to previous checks
     assert r is not None
 
-    if plot.backend.name == 'pylab':
-        assert '<summary>FitContour</summary>' in r
-        assert '<svg ' in r
+    if plot_backend_is("pylab"):
+        assert "<summary>FitContour</summary>" in r
+        assert "<svg " in r
         return
 
-    assert '<summary>DataContour (' in r
-    assert '<summary>ModelContour (' in r
+    assert "<summary>DataContour (" in r
+    assert "<summary>ModelContour (" in r
     assert '<div class="dataval">n n</div>' in r
     assert '<div class="dataval">Model</div>' in r
 
@@ -278,12 +283,12 @@ def test_intproj(old_numpy_printing, override_plot_backend):
     r = p._repr_html_()
     assert r is not None
 
-    if plot.backend.name == 'pylab':
-        assert '<summary>IntervalProjection</summary>' in r
-        assert '<svg ' in r
+    if plot_backend_is("pylab"):
+        assert "<summary>IntervalProjection</summary>" in r
+        assert "<svg " in r
         return
 
-    assert '<summary>IntervalProjection (8)</summary>' in r
+    assert "<summary>IntervalProjection (8)</summary>" in r
 
     assert '<div class="dataname">x</div><div class="dataval">[ 1.        1.555556  2.111111  2.666667  3.222222  3.777778  4.333333  4.888889\n  5.444444  6.      ]</div>' in r
     assert '<div class="dataname">nloop</div><div class="dataval">10</div>' in r
@@ -313,21 +318,29 @@ def test_regproj(old_numpy_printing, override_plot_backend):
     r = p._repr_html_()
     assert r is not None
 
-    if plot.backend.name == 'pylab':
-        assert '<summary>RegionProjection</summary>' in r
-        assert '<svg ' in r
+    if plot_backend_is("pylab"):
+        assert "<summary>RegionProjection</summary>" in r
+        assert "<svg " in r
         return
 
-    print(r)
-    assert '<summary>RegionProjection (13)</summary>' in r
+    assert "<summary>RegionProjection (13)</summary>" in r
 
-    assert '<div class="dataname">parval0</div><div class="dataval">-0.5315772076542427</div>' in r
-    assert '<div class="dataname">parval1</div><div class="dataval">0.5854611101216837</div>' in r
+    # Issue #1372 shows that the numbers here can depend on the platform; as
+    # this test is not about whether the fit converged to the same solution
+    # the tests are very basic. An alternative would be to just place
+    # the values from the fit object into the strings, but then there is
+    # the problem that this test currently requires old_numpy_printing,
+    # so the results would not necessarily match.
+    #
+    assert '<div class="dataname">parval0</div><div class="dataval">-0.5' in r
+    assert '<div class="dataname">parval1</div><div class="dataval">0.5' in r
     assert '<div class="dataname">sigma</div><div class="dataval">(1, 2, 3)</div>' in r
 
-    assert '<div class="dataname">y</div><div class="dataval">[ 306.854444  282.795953  259.744431  237.699877  216.662291  196.631674\n' in r
+    # These values may depend on the platform so only very-limited check.
+    #
+    assert '<div class="dataname">y</div><div class="dataval">[ 30' in r
+    assert '<div class="dataname">levels</div><div class="dataval">[  3.6' in r
 
-    assert '<div class="dataname">levels</div><div class="dataval">[  3.606863   7.491188  13.140272]</div>' in r
     assert '<div class="dataname">min</div><div class="dataval">[-2, -1]</div>' in r
     assert '<div class="dataname">max</div><div class="dataval">[2, 2]</div>' in r
 
